using System.Collections.Generic;

namespace Syntriax.Engine.Core.Abstract;

/// <summary>
/// Represents a game world responsible for managing <see cref="IHierarchyObject"/>s.
/// </summary>
public interface IGameManager : IEntity, IEnumerable<IHierarchyObject>
{
    /// <summary>
    /// Event triggered when <see cref="Update(double)"/> is called on the <see cref="IGameManager"/>.
    /// </summary>
    event UpdateEventHandler? OnUpdate;

    /// <summary>
    /// Event triggered when <see cref="PreDraw"/> is called on the <see cref="IGameManager"/>.
    /// </summary>
    event PreDawEventHandler? OnPreDraw;

    /// <summary>
    /// Event triggered when a <see cref="IHierarchyObject"/> is registered to the <see cref="IGameManager"/>.
    /// </summary>
    event HierarchyObjectRegisteredEventHandler? OnHierarchyObjectRegistered;

    /// <summary>
    /// Event triggered when a <see cref="IHierarchyObject"/> is unregistered from the <see cref="IGameManager"/>.
    /// </summary>
    event HierarchyObjectUnRegisteredEventHandler? OnHierarchyObjectUnRegistered;

    /// <summary>
    /// Contains time data related to this <see cref="IGameManager"/>.
    /// </summary>
    EngineTime Time { get; }

    /// <summary>
    /// Gets a read-only list of <see cref="IHierarchyObject"/>s managed by the <see cref="IGameManager"/>.
    /// </summary>
    IReadOnlyList<IHierarchyObject> HierarchyObjects { get; }

    /// <summary>
    /// Registers an <see cref="IHierarchyObject"/> to the <see cref="IGameManager"/>.
    /// </summary>
    /// <param name="hierarchyObject">The <see cref="IHierarchyObject"/> to register.</param>
    void Register(IHierarchyObject hierarchyObject);

    /// <summary>
    /// Instantiates a <see cref="IHierarchyObject"/> of type T with the given arguments and registers it to the <see cref="IGameManager"/>.
    /// </summary>
    /// <typeparam name="T">The type of <see cref="IHierarchyObject"/> to instantiate.</typeparam>
    /// <param name="args">Constructor parameters for the given type of <see cref="IHierarchyObject"/>.</param>
    /// <returns>The instantiated <see cref="IHierarchyObject"/>.</returns>
    T InstantiateHierarchyObject<T>(params object?[]? args) where T : class, IHierarchyObject;

    /// <summary>
    /// Removes an <see cref="IHierarchyObject"/> from the <see cref="IGameManager"/>.
    /// </summary>
    /// <param name="hierarchyObject">The <see cref="IHierarchyObject"/> to remove.</param>
    void Remove(IHierarchyObject hierarchyObject);

    /// <summary>
    /// Updates the <see cref="IGameManager"/> with the given delta time.
    /// </summary>
    /// <param name="engineTime">Delta time.</param>
    void Update(EngineTime engineTime);

    /// <summary>
    /// Performs operations that should be done before the draw calls.
    /// </summary>
    void PreDraw();

    delegate void UpdateEventHandler(IGameManager sender, EngineTime engineTime);
    delegate void PreDawEventHandler(IGameManager sender);

    delegate void HierarchyObjectRegisteredEventHandler(IGameManager sender, IHierarchyObject hierarchyObjectRegistered);
    delegate void HierarchyObjectUnRegisteredEventHandler(IGameManager sender, IHierarchyObject hierarchyObjectUnregistered);
}
